#!/usr/bin/env python3
# Copyright (C) 2017 Evgeny Golyshev <eugulixes@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Utility intended to fetch the image attributes from the specified pieman.yml
file.
"""

import os
import sys
from argparse import ArgumentParser

from pieman import attrs


def fail(message):
    """Writes the specified message to stderr and exits with a non-zero exit
    code. """
    sys.stderr.write(message + '\n')
    sys.exit(1)


def main():
    """The main entry point. """
    parser = ArgumentParser()
    parser.add_argument('-f', '--file', dest='file', default='pieman.yaml',
                        help='path to a YAML file which describes the target '
                             'image')
    parser.add_argument('root', nargs='*')
    args = parser.parse_args()

    if not os.path.isfile(args.file):
        fail('{} does not exist'.format(args.file))

    attributes_list = attrs.AttributesList(args.file)

    try:
        attr = attributes_list.get_attribute(args.root)
    except attrs.RootDoesNotExist:
        fail('There is no root named {}'.format(args.root))
    except attrs.AttributeDoesNotExist as exc:
        fail(str(exc))
    except attrs.UnknownAttribute:
        fail('{} attribute is unknown'.format(args.root[-1]))

    try:
        attr.echo()
    except attrs.UnprintableType:
        fail('{} attribute is not supposed to be printed'.format(args.root[-1]))


if __name__ == "__main__":
    main()



#!/usr/bin/python3
# Copyright (C) 2018 Evgeny Golyshev <eugulixes@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Utility intended to calculate the size of the chroot environments.
The main motivation to create a substitution for du from GNU Coreutils is that
it can sometimes provide inaccurate result. For example, directory size may
vary depending on when du is called -- before or after transferring the
directory to an image.
"""

import math
import os
import sys
from argparse import ArgumentParser


def get_tree_size(path, block_size=4096):
    """Returns total size of files and number of files in the specified
    directory.
    """
    total_size = items_number = 0

    # os.walk skips the symbolic links that resolve to directories, not
    # counting them at all. It has a great impact on the end result, so we need
    # a different way to solve the task.
    for dir_entry in fault_tolerant_scandir(path):
        items_number += 1

        if os.path.islink(dir_entry.path):
            total_size += block_size
        elif os.path.isdir(dir_entry.path):
            ret = get_tree_size(dir_entry.path)
            total_size += ret[0] + block_size
            items_number += ret[1]
        elif os.path.isfile(dir_entry.path):
            file_size = os.path.getsize(dir_entry.path)
            if file_size > block_size:
                total_size += file_size
            else:
                total_size += block_size

    return total_size, items_number


def fault_tolerant_scandir(path):
    """Wrapper for ``os.scandir`` which is supposed to write to stderr if there
    is some problems instead of throwing exceptions.
    """

    try:
        return os.scandir(path)
    except PermissionError:
        sys.stderr.write('Permission denied when trying to figure out '
                         'the size of {}\n'.format(path))

    return []


def main():
    """The main entry point. """

    parser = ArgumentParser()
    parser.add_argument("-b", "--block-size", default=4096, type=int,
                        help="block size", metavar="SIZE")
    parser.add_argument("-m", "--size-in-megabytes", action="store_true",
                        help="return total size in megabytes", dest="megabytes")
    parser.add_argument('directory')
    args = parser.parse_args()

    if not os.path.isdir(args.directory):
        sys.stderr.write('The {} directory does not exist\n'.format(args.directory))
        sys.exit(1)

    total_size, items_number = get_tree_size(args.directory, args.block_size)

    if args.megabytes:
        total_size = math.ceil(total_size / 1024 / 1024)

    print('Items number: {}'.format(items_number))
    print('Total size: {}'.format(total_size))


if __name__ == '__main__':
    main()



#!/usr/bin/python3
# Copyright (C) 2019 Evgeny Golyshev <eugulixes@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Utility intended to check if the Redis server is available.
It returns a zero exit status if everything is ok and non-zero exit status if
there is a problem to connect to the specified Redis server.
"""

import sys
from argparse import ArgumentParser

from redis.exceptions import RedisError

from pieman import util


def main():
    """The main entry point. """

    parser = ArgumentParser()
    parser.add_argument('-H', '--redis-host', default='127.0.0.1',
                        help='Redis server host', metavar='REDIS_HOST')
    parser.add_argument('-P', '--redis-port', default='6379',
                        help='server pid file name', metavar='REDIS_PORT',
                        type=int)

    args = parser.parse_args()

    try:
        util.connect_to_redis(args.redis_host, args.redis_port)
    except RedisError:
        sys.exit(1)


if __name__ == '__main__':
    main()



#!/usr/bin/python3
# Copyright (C) 2019 Evgeny Golyshev <eugulixes@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""BSC server. BSC stands for Build Status Codes.
The server is very simple and supposed to handle one client at a time.
"""

import logging
import os
import socket
import sys
from argparse import ArgumentParser

from daemon import DaemonContext, pidfile
from redis.exceptions import RedisError

from pieman import codes, util

BUFSIZE = 4096

DAEMON_NAME = 'bscd'

LOGGER = logging.getLogger(DAEMON_NAME)


def test_write_perm(dir_name):
    """Tests if the current process has enough permissions to create files
    in the specified directory.
    """

    dir_name = dir_name if dir_name else './'
    if not os.access(dir_name, os.W_OK):
        LOGGER.fatal('%s is not writable for the process. Make sure that the '
                     'directory exists and the process has enough permissions '
                     'to create files there.', dir_name)
        sys.exit(1)


class BSCServer:
    """Class implementing the build status codes server. """

    def __init__(self, unix_socket_name, channel_name, redis_host='127.0.0.1',
                 redis_port=6379):
        self._channel_name = channel_name
        self._conn = None
        self._redis_conn = util.connect_to_redis(redis_host, redis_port)
        self._server = None
        self._unix_socket_name = unix_socket_name

    #
    # Private methods
    #

    def _accept(self):
        """Accepts a connection. """

        self._conn, _ = self._server.accept()

        LOGGER.debug('Client connected')

    def _close_server(self):
        """Closes the specified server and removes the corresponding Unix
        domain socket file.
        """
        sock_name = self._server.getsockname()

        self._server.close()

        if os.path.exists(sock_name):
            os.unlink(sock_name)

    def _run_loop(self):
        """Runs the loop which handles requests from the client. """

        status_codes = [name for name in dir(codes) if name.endswith('_CODE')]

        handlers = {getattr(codes, code): self.send_code_to_channel
                    for code in status_codes}

        handlers[b'EXIT'] = self.exit_server

        self._accept()

        while True:
            request = self._conn.recv(BUFSIZE)

            if not request:
                LOGGER.debug('Client disconnected')

                self._accept()

                # Not to pass an empty request further, skip the current
                # iteration
                continue

            try:
                handler = handlers[request]
            except KeyError:
                LOGGER.fatal('Unknown request %s', request)
                continue

            handler(request)

    #
    # Public methods
    #

    def exit_server(self, _code):
        """Shuts down the server. """

        LOGGER.info('Shutting down the server since the EXIT request was sent')
        self._close_server()
        sys.exit(0)

    def run(self):
        """Runs the server. """

        self._server = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self._server.bind(self._unix_socket_name)

        self._server.listen(1)

        try:
            self._run_loop()
        except KeyboardInterrupt:
            LOGGER.info('Shutting down the server since the signal was '
                        'generated by Ctrl-C')
            self._close_server()
            sys.exit(1)

    def send_code_to_channel(self, code):
        """Sends the specified build code to the Redis channel. """
        LOGGER.info('Handling %s', code)
        self._redis_conn.publish(self._channel_name, code)


def main():
    """The main entry point. """

    parser = ArgumentParser()
    parser.add_argument('-a', '--unix-socket-name',
                        default='/var/run/{}.sock'.format(DAEMON_NAME),
                        help='Unix domain socket file name',
                        metavar='UNIX_SOCKET_NAME')
    parser.add_argument('-d', '--daemonize', action='store_true',
                        help='daemonize the server')
    parser.add_argument('-H', '--redis-host', default='127.0.0.1',
                        help='Redis server host', metavar='REDIS_HOST')
    parser.add_argument('-l', '--log-file-prefix', default='',
                        help='path prefix for log files',
                        metavar='LOG_FILE_PREFIX')
    parser.add_argument('-n', '--channel-name', default='build_status_codes',
                        help='channel which is used for publishing build '
                             'status codes to',
                        metavar='CHANNEL_NAME')
    parser.add_argument('-p', '--pid',
                        default='/var/run/{}.pid'.format(DAEMON_NAME),
                        help='server pid file name',
                        metavar='PID_FILE_NAME')
    parser.add_argument('-P', '--redis-port', default='6379',
                        help='server pid file name', metavar='REDIS_PORT',
                        type=int)
    parser.add_argument('-v', '--verbose', action='store_true',
                        help='turn on verbose mode')

    args = parser.parse_args()

    util.init_logger(LOGGER, logging.DEBUG if args.verbose else logging.INFO,
                     args.log_file_prefix)

    if os.path.exists(args.unix_socket_name):
        LOGGER.fatal('Address already in use')
        sys.exit(1)
    else:
        test_write_perm(os.path.dirname(args.unix_socket_name))

    if args.log_file_prefix:
        test_write_perm(os.path.dirname(args.log_file_prefix))
        LOGGER.debug('All log records will be forwarded to %s',
                     args.log_file_prefix)
    else:
        LOGGER.debug('No log file will be created')

    try:
        bscd = BSCServer(args.unix_socket_name, args.channel_name,
                         redis_host=args.redis_host,
                         redis_port=args.redis_port)
    except RedisError:
        LOGGER.fatal('Could not connect to Redis')
        sys.exit(1)

    if args.daemonize:
        test_write_perm(os.path.dirname(args.pid))

        LOGGER.debug('Server will be daemonized')

        # Preserve the fds related to the logger to prevent closing when
        # daemonizing the server.
        preserved_fds = []
        for handler in LOGGER.handlers:
            if hasattr(handler, 'stream'):
                preserved_fds.append(handler.stream.fileno())

        with DaemonContext(files_preserve=preserved_fds,
                           pidfile=pidfile.TimeoutPIDLockFile(args.pid),
                           working_directory=os.getcwd()):
            bscd.run()
    else:
        bscd.run()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# Copyright (C) 2019 Evgeny Golyshev <eugulixes@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Utility intended for checking if the specified WPA passphrase is valid. The
following two checks were borrowed from wpa_supplicant/wpa_passphrase.c.
The utility exits with:
* 0 when everything is fine;
* 1 when the length of the specified passphrase is not between 8 and 63
    characters;
* 2 when the specified passphrase contains special characters.
"""

import argparse
import sys


SPECIAL_CHARACTERS = [chr(n) for n in range(32)] + ['\x7f']


def main():
    """The main entry point. """

    parser = argparse.ArgumentParser()
    parser.add_argument('passphrase', help='WPA passphrase which is going to '
                                           'be passed to wpa_passphrase')
    args = parser.parse_args()

    passphrase_len = len(args.passphrase)
    if passphrase_len < 8 or passphrase_len > 63:
        sys.exit(1)

    for char in args.passphrase:
        if char in SPECIAL_CHARACTERS:
            sys.exit(2)


if __name__ == '__main__':
    main()



# Copyright (C) 2019 Evgeny Golyshev <eugulixes@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Miscellaneous utility functions. """

import logging
import os
import sys
from urllib.request import urlretrieve

import redis

LOGGING_FORMATTER = '%(asctime)s %(levelname)-5.5s %(message)s'


def _reporthook(chunk_number, buffer_size, total_size):
    """It must accept three numeric parameters:
    - a chunk number;
    - the maximum size chunks are read in;
    - the total size of the download (-1 if unknown).
    """

    readsofar = chunk_number * buffer_size
    readsofar = total_size if readsofar > total_size else readsofar
    if total_size:
        percent = readsofar * 100 / total_size
        status = '\r{:>5.1f}% {:>{n}} / {}'.format(
            percent, readsofar, total_size, n=len(str(total_size)))
        sys.stderr.write(status)
    else:  # total size is unknown
        sys.stderr.write('\rread {}'.format(readsofar))


def connect_to_redis(host, port):
    """Connects to the specified Redis server. The function raises on of the
    exceptions derived from redis.exceptions.RedisError in case of a problem.
    """
    conn = redis.StrictRedis(host=host, port=port)
    conn.ping()

    return conn


def download(url, dst, quiet=False):
    """Downloads the specified document from the Web,
    (optionally) displaying a progress bar.
    """
    _, msg = urlretrieve(url, dst + '.part',
                         _reporthook if not quiet else None)

    os.rename(dst + '.part', dst)

    size = os.stat(dst).st_size
    if not quiet:
        sys.stderr.write('\n')
        if int(msg['Content-Length']) == size:
            filename = os.path.basename(dst)
            sys.stderr.write('{} was downloaded successfully\n'
                             .format(filename))


def init_logger(logger, log_level, log_file_prefix='',
                logging_formatter=LOGGING_FORMATTER):
    """Initializes the logger. """

    formatter = logging.Formatter(logging_formatter)
    logger.setLevel(log_level)

    streaming_handler = logging.StreamHandler()
    streaming_handler.setLevel(log_level)
    streaming_handler.setFormatter(formatter)
    logger.addHandler(streaming_handler)

    if log_file_prefix:
        file_handler = logging.FileHandler(log_file_prefix)
        file_handler.setFormatter(formatter)
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)



#!/usr/bin/env python3
# Copyright (C) 2019 Evgeny Golyshev <eugulixes@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Utility intended for rendering Jinja2 templates.
The utility exits with:
* 0 when everything is fine;
* 1 when the specified template does not exist;
* 2 when the directory of the specified result path does not exist.
"""

import argparse
import os
import sys

from jinja2 import Environment, FileSystemLoader


def getenv(value, key):
    """Acts like any Jinja2 filter and gets the value of the specified
    environment variable.
    Note that the filter returns the native Python boolean if the environment
    variable contains 'false' . It's necessary for using the values in
    conditions.
    """

    val = os.getenv(key, value)
    return False if val == 'false' else val


def main():
    """The main entry point. """

    parser = argparse.ArgumentParser()
    parser.add_argument('template_path', help='path to the template to be '
                                              'rendered')
    parser.add_argument('result_path', help='path to the result of rendering')
    args = parser.parse_args()

    template_dir = os.path.dirname(args.template_path)
    template_name = os.path.basename(args.template_path)
    result_dir = os.path.dirname(args.result_path)

    if not os.path.isfile(args.template_path):
        sys.exit(1)

    if not os.path.isdir(result_dir):
        sys.exit(2)

    env = Environment(loader=FileSystemLoader(template_dir))
    env.filters['getenv'] = getenv

    template = env.get_template(template_name)
    output = template.render()

    with open(args.result_path, 'w') as outfile:
        outfile.write(output)


if __name__ == '__main__':
    main()



#!/usr/bin/python3
# Copyright (C) 2018 Evgeny Golyshev <eugulixes@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Utility intended to fetch the latest version of the apk-tools-static package
in the specified version of Alpine Linux. The point is that the
apk-tools-static version is not frozen in the stable release of Alpine Linux
and may vary.
"""

import os
import sys
import time
from argparse import ArgumentParser
from html.parser import HTMLParser
from urllib.error import URLError, HTTPError
from urllib.parse import urljoin
from urllib.request import urlopen


ARCH = 'armhf'

ALPINE_VERSION = '3.9'

MIRROR = 'http://dl-cdn.alpinelinux.org'

NAP = 1

RETRIES_NUMBER = 5


class CustomHTMLParser(HTMLParser):  # pylint: disable=abstract-method
    """Simplified HTML parser to find the apk-tools-static version on the
    specified page.
    """

    def __init__(self, content):
        HTMLParser.__init__(self, convert_charrefs=True)

        self._apk_tools_version = None
        self._content = content

    def get_apk_tools_version(self):
        """Returns the apk-tools-static version. """
        self.feed(self._content)
        return self._apk_tools_version

    def handle_starttag(self, tag, attrs):
        if tag == 'a':
            for name, value in attrs:
                package_name = 'apk-tools-static'
                if name == 'href' and value.startswith(package_name):
                    prefix = len(package_name) + 1
                    suffix = len('.apk')
                    self._apk_tools_version = value[prefix:-suffix]
                    break


def main():
    """The main entry point. """

    parser = ArgumentParser()
    parser.add_argument('--alpine-version', default=ALPINE_VERSION,
                        help='alpine version', metavar='ALPINE_VERSION')
    parser.add_argument('--arch', default=ARCH,
                        help='target architecture', metavar='ARCH')
    parser.add_argument('--mirror', default=MIRROR,
                        help='mirror', metavar='MIRROR')
    args = parser.parse_args()

    address = urljoin(args.mirror,
                      os.path.join('alpine', 'v' + args.alpine_version,
                                   'main', args.arch))

    content = b''
    for attempt in range(1, RETRIES_NUMBER + 1):
        try:
            content = urlopen(address).read()
            break
        except HTTPError as exc:
            sys.stderr.write('{}: request failed (error code {})\n'.
                             format(sys.argv[0], exc.code))
        except URLError as exc:
            sys.stderr.write('{}: {}\n'.format(sys.argv[0], exc.reason))

        if attempt != RETRIES_NUMBER:
            sys.stderr.write('Retrying in {} seconds...\n'.format(NAP))
            time.sleep(NAP)

    if content == b'' and attempt == RETRIES_NUMBER:
        sys.stderr.write('Could not request {} after {} attempts\n'.
                         format(address, RETRIES_NUMBER))
        sys.exit(1)

    parser = CustomHTMLParser(content.decode('utf8'))
    apk_tools_version = parser.get_apk_tools_version()
    if not apk_tools_version:
        sys.stderr.write('Could not get apk tools version\n')
        sys.exit(1)

    print(apk_tools_version)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# Copyright (C) 2019 Evgeny Golyshev <eugulixes@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Utility intended for checking if the dependency environment variables,
passed as the 2nd, 3rd, etc arguments to the utility, are set to true (if bool)
or simply specified (in other cases) when the dependent environment variable,
passed as the first argument to the utility, is set to true (if bool) or simply
specified (in other cases).
"""

import argparse
import os
import sys


def main():
    """The main entry point. """

    parser = argparse.ArgumentParser()
    parser.add_argument('var', help='name of dependent environment variable')
    parser.add_argument('dependency_var', help='name of environment variable '
                                               'var1 depends on')
    args = parser.parse_args()

    var = os.getenv(args.var, None)
    dependency_var = os.getenv(args.dependency_var, None)

    var = '' if var == 'false' else var
    dependency_var = '' if dependency_var == 'false' else dependency_var
    if not var:
        sys.exit(0)

    if not dependency_var:
        sys.exit(1)


if __name__ == '__main__':
    main()



#!/usr/bin/python3
# Copyright (C) 2019 Evgeny Golyshev <eugulixes@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""BSC client. BSC stands for Build Status Codes. """

import os
import socket
import sys
from argparse import ArgumentParser

import bscd


def main():
    """The main entry point. """

    parser = ArgumentParser()
    parser.add_argument('-a', '--unix-socket-name',
                        default='/var/run/{}.sock'.format(bscd.DAEMON_NAME),
                        help='Unix domain socket file name',
                        metavar='UNIX_SOCKET_NAME')
    parser.add_argument('-H', '--redis-host', default='127.0.0.1',
                        help='Redis server host', metavar='REDIS_HOST')
    parser.add_argument('-P', '--redis-port', default='6379',
                        help='server pid file name', metavar='REDIS_PORT',
                        type=int)
    parser.add_argument('request', metavar='REQUEST',
                        help='request to be sent to bsc server')

    args = parser.parse_args()

    if not os.path.exists(args.unix_socket_name):
        sys.stderr.write('{} does not exist\n'.format(args.unix_socket_name))
        sys.exit(1)

    client = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)

    try:
        client.connect(args.unix_socket_name)
    except ConnectionRefusedError:
        sys.stderr.write('Connection refused\n')
        sys.exit(1)

    client.send(args.request.encode('utf-8'))
    client.close()


if __name__ == '__main__':
    main()



# Copyright (C) 2019 Evgeny Golyshev <eugulixes@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Module intended for parsing the build_status_codes text file. """

import os
import sys


BUILD_STATUS_CODES = os.path.join(os.path.dirname(__file__),
                                  'build_status_codes')

MODULE = sys.modules[__name__]

LINES = [line.rstrip('\n').strip() for line in open(BUILD_STATUS_CODES)]
for line in LINES:
    if line and not line.startswith('#'):
        nam, val = line.split('=')
        setattr(MODULE, nam, val.encode())  # values must be bytes



""" Script for building the Pieman package. """

from setuptools import setup

try:
    import pypandoc
    LONG_DESCRIPTION = pypandoc.convert('README.md', 'rst')
except ImportError:
    LONG_DESCRIPTION = ('Utilities written in Python which are used by '
                        'Pieman, script for creating custom OS images for '
                        'single-board computers.')


setup(name='pieman',
      version='0.12.0',
      description='Pieman package',
      long_description=LONG_DESCRIPTION,
      url='https://github.com/tolstoyevsky/pieman',
      author='Evgeny Golyshev',
      maintainer='Evgeny Golyshev',
      maintainer_email='eugulixes@gmail.com',
      license='https://gnu.org/licenses/gpl-3.0.txt',
      scripts=[
          'bin/apk_tools_version.py',
          'bin/bsc.py',
          'bin/bscd.py',
          'bin/check_mutually_exclusive_params.py',
          'bin/check_redis.py',
          'bin/check_wpa_passphrase.py',
          'bin/depend_on.py',
          'bin/du.py',
          'bin/image_attrs.py',
          'bin/render.py',
      ],
      packages=['pieman'],
      include_package_data=True,
      data_files=[('pieman', ['pieman/build_status_codes'])],
      install_requires=[
          'Jinja2==2.10.1',
          'PyYAML==5.1',
      ])



#!/usr/bin/env python3
# Copyright (C) 2018 Denis Gavrilyuk <karpa4o4@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Utility intended for checking if two specified environment variables are
defined. If so, the utility exits with a non-zero code. Otherwise it exits
normally.
"""

import argparse
import os
import sys


def main():
    """The main entry point. """

    parser = argparse.ArgumentParser()
    parser.add_argument('var1', help='first environment variable name')
    parser.add_argument('var2', help='second environment variable name')
    args = parser.parse_args()

    var1 = os.getenv(args.var1, None)
    var2 = os.getenv(args.var2, None)

    var1 = '' if var1 == 'false' else var1
    var2 = '' if var2 == 'false' else var2
    if var1 and var2:
        sys.exit(1)


if __name__ == '__main__':
    main()



# Copyright (C) 2017 Evgeny Golyshev <eugulixes@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Module intended to simplify working with pieman.yml files. """

import yaml


TYPES_SCHEME = {
    'repos': (list, ),
    'codename': (str, ),
    'base': (list, str),
    'includes': (list, str),
    'boot': (list, str),
    'params': (list, ),
    'kernel': {
        'package': (str, ),
        'rebuild': (bool, ),
        'patches': (str, ),
    },
}


class AttributeDoesNotExist(Exception):
    """Exception raised when attempting to get an attribute which does not
    exist. """
    def __init__(self, par_name, cur_name):
        message = ('{} does not have attribute {}'.format(par_name, cur_name))
        Exception.__init__(self, message)


class RootDoesNotExist(Exception):
    """Exception raised when attempting to get the root which does not
    exist. """


class UnknownAttribute(Exception):
    """Exception raised when attempting to get an attribute which may exist
    but is not mentioned in the specification. """


class UnprintableType(Exception):
    """Exception raised when attempting to print an attribute the type of which
    is neither str nor list. """


class Attribute:  # pylint: disable=too-few-public-methods
    """Class representing a single attribute. """

    def __init__(self, attribute, attribute_type):
        self.attribute, self._attribute_type = attribute, attribute_type

    def echo(self):
        """Writes the value of the attribute to stdout or raises
        `UnprintableType` if the attribute type is neither str nor list.
        """
        if list in self._attribute_type or str in self._attribute_type:
            if isinstance(self.attribute, str):
                self.attribute = [self.attribute]

            for line in self.attribute:
                print(line)
        else:
            raise UnprintableType


class AttributesList:  # pylint: disable=too-few-public-methods
    """Class implementing the interface for working with pieman.yml files. """

    def __init__(self, attributes_file):
        with open(attributes_file, 'r') as infile:
            self._attributes = yaml.load(infile, Loader=yaml.FullLoader)

    def get_attribute(self, attributes_chain):
        """Gets the value of the attribute. To get the value the full path to
        the attribute must be specified starting with the root.
        """

        if attributes_chain:
            par_name = attributes_chain[0]
            try:
                cur_attribute = self._attributes[par_name]
            except KeyError:
                raise RootDoesNotExist

            cur_type = TYPES_SCHEME

            for attribute_name in attributes_chain[1:]:
                try:
                    cur_attribute = cur_attribute[attribute_name]
                except KeyError:
                    raise AttributeDoesNotExist(par_name, attribute_name)

                try:
                    cur_type = cur_type[attribute_name]
                except KeyError:
                    raise UnknownAttribute

                par_name = attribute_name

            if isinstance(cur_type, dict):
                cur_type = (dict, )

            return Attribute(cur_attribute, cur_type)

        return None
